package at.ac.uibk.gitsearch.service.dto;

import java.util.Date;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonValue;

public class SearchResultsDTO {

	@JsonFormat(shape = JsonFormat.Shape.OBJECT)
	public enum ExerciseType {
		PROGRAMMING_EXERCISE("programming exercise"), EXERCISE("exercise"), COLLECTION("collection"), OTHER("other");

		ExerciseType(String externalName) {
			this.externalName = externalName;
		}

		private String externalName;

		@JsonValue
		public String getExternalName() {
			return externalName;
		}
	}

	public static class Person {
		private String name;
		private String affiliation;
		private String email;

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public String getAffiliation() {
			return affiliation;
		}

		public void setAffiliation(String affiliation) {
			this.affiliation = affiliation;
		}

		public String getEmail() {
			return email;
		}

		public void setEmail(String email) {
			this.email = email;
		}

		@Override
		public String toString() {
			return "Person: { name: " + this.name + ", affiliation: " + this.affiliation + ", email: " + this.email
					+ "}";
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((affiliation == null) ? 0 : affiliation.hashCode());
			result = prime * result + ((email == null) ? 0 : email.hashCode());
			result = prime * result + ((name == null) ? 0 : name.hashCode());
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			Person other = (Person) obj;
			if (affiliation == null) {
				if (other.affiliation != null)
					return false;
			} else if (!affiliation.equals(other.affiliation))
				return false;
			if (email == null) {
				if (other.email != null)
					return false;
			} else if (!email.equals(other.email))
				return false;
			if (name == null) {
				if (other.name != null)
					return false;
			} else if (!name.equals(other.name))
				return false;
			return true;
		}
	}

	public enum GitProjectVisibility {
		PRIVATE("private"), PUBLIC("public"), INTERNAL("internal");

		private String externalName;

		private GitProjectVisibility(String externalName) {
			this.externalName = externalName;
		}

		@JsonValue
		public String getExternalName() {
			return externalName;
		}

	}

	private List<SearchResultDTO> searchResult;

	/**
	 *
	 * @return the list of search results on this page, starting with pageStartIndex
	 */
	public List<SearchResultDTO> getSearchResult() {
		return searchResult;
	}

	public void setSearchResult(List<SearchResultDTO> searchResult) {
		this.searchResult = searchResult;
	}

	long hitCount;

	long pageStartIndex;

	public long getPageStartIndex() {
		return pageStartIndex;
	}

	public SearchResultsDTO(List<SearchResultDTO> searchResult, long hitCount, long pageStartIndex) {
		this.searchResult = searchResult;
		this.hitCount = hitCount;
		this.pageStartIndex = pageStartIndex;
	}

	public SearchResultsDTO() {

	}

	/**
	 * returns the number of all hits.
	 *
	 * @return
	 */
	public long getHitCount() {
		return hitCount;
	}

	public void setHitCount(long hitCount) {
		this.hitCount = hitCount;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("SearchResultsDTO:");
		sb.append(System.lineSeparator());
		sb.append("hit count: ");
		sb.append(this.hitCount);
		sb.append(System.lineSeparator());
		sb.append("page start index: ");
		sb.append(this.pageStartIndex);
		sb.append(System.lineSeparator());
		sb.append("list of SearchResultsDTOs:");
		sb.append(System.lineSeparator());
		for (SearchResultDTO result : this.searchResult) {
			sb.append(result);
			sb.append(System.lineSeparator());
			sb.append(System.lineSeparator());
		}
		return sb.toString();
	}
}
